'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var loaderUtils = require('loader-utils');
var parser = require('@babel/parser');
var BabelTypeModule = require('@babel/types');
var generator = _interopDefault(require('@babel/generator'));
var traverse = _interopDefault(require('@babel/traverse'));
var toPath = _interopDefault(require('lodash/topath'));
var path = _interopDefault(require('path'));
var glob = _interopDefault(require('glob'));
var fs = _interopDefault(require('fs'));

// 	REACT_APP_PARAM_ONE: 'I AM PARAM ONE',
// 	REACT_APP_PARAM_TWO: 'I AM PARAM TWO',
// 	REACT_APP_PARAM_THREE: 'I AM PARAM THREE',
// };
// const read = fs.readFileSync("./testbuildfile.js", "utf8");
// console.log("read", read);

var defaultOptions = {
  globOptions: {
    cwd: __dirname
  },
  pattern: '*',
  envVar: {},
  debug: "development" !== 'production',
  destination: __dirname,
  updateInline: true
};

function injectEnv(options) {
  var _options$globOptions = options.globOptions,
      globOptions = _options$globOptions === void 0 ? defaultOptions.globOptions : _options$globOptions,
      _options$envVar = options.envVar,
      envVar = _options$envVar === void 0 ? defaultOptions.envVar : _options$envVar,
      _options$debug = options.debug,
      debug = _options$debug === void 0 ? defaultOptions.debug : _options$debug,
      _options$pattern = options.pattern,
      pattern = _options$pattern === void 0 ? defaultOptions.pattern : _options$pattern,
      _options$updateInline = options.updateInline,
      updateInline = _options$updateInline === void 0 ? defaultOptions.updateInline : _options$updateInline;
  var _globOptions$cwd = globOptions.cwd,
      cwd = _globOptions$cwd === void 0 ? defaultOptions.globOptions.cwd : _globOptions$cwd;
  var envFromParamStore = envVar;
  var destination = options.destination || defaultOptions.destination;

  if (updateInline) {
    destination = cwd;
  }

  var Files = glob.sync(pattern, {
    cwd: cwd
  });

  if (debug) {
    console.log('Files are ==>', Files); // got the files, not need to replace
  }

  Files.forEach(function (fileName) {
    var source = '';
    var pathToCheckAndUpdate = path.join(cwd, fileName);

    if (debug) {
      console.log('pathToCheckAndUpdate', pathToCheckAndUpdate);
    }

    var read = fs.readFileSync(pathToCheckAndUpdate, 'utf8');
    var divider = '$$_INTERNAL__';
    source = read;
    var noOfMatches = source.match(/\$\$_INTERNAL__/g);

    if (noOfMatches && noOfMatches.length > 1) {
      var changed = false;
      var splitSource = source.split(divider);
      source = splitSource.reduce(function (acc, elem) {
        var stringToBeReplaced = elem;

        if (acc.remove) {
          // need to replace the extra ) added for concat
          stringToBeReplaced = stringToBeReplaced.replace(/\)/, ''); // }

          acc.remove = false;
        }

        var ifElementStartsWithProcessEnv = elem.startsWith('process.env.');
        var indexOfMathRandomStart = elem.search('MATH_RANDOM_START');

        if (indexOfMathRandomStart !== -1 && ifElementStartsWithProcessEnv) {
          var temp = elem.slice(indexOfMathRandomStart);
          stringToBeReplaced = stringToBeReplaced.replace(temp, '');
          var splittedString = stringToBeReplaced.split('.');
          var firstVariable = splittedString[0],
              secondVariable = splittedString[1],
              ThirdVariable = splittedString[2];
          var isValidFirstVariable = firstVariable.trim() === 'process';
          var isValidSecondVariable = secondVariable.trim() === 'env';
          var paramValue = // @ts-ignore
          envFromParamStore[ThirdVariable.trim()] || undefined;
          var isValidThirdVariable = paramValue !== undefined;

          if (isValidFirstVariable && isValidSecondVariable && isValidThirdVariable) {
            if (paramValue !== undefined) {
              acc.result.push(paramValue);
            }

            var countOfOpeningBrackets = temp.match(/\(/g);
            var countClosingBrackets = temp.match(/\)/g);

            if (countOfOpeningBrackets && countClosingBrackets && countOfOpeningBrackets.length > countClosingBrackets.length) {
              acc.remove = true;
            }

            changed = true;
            return acc;
          }
        }

        acc.result.push(stringToBeReplaced);
        acc.remove = false;
        return acc;
      }, {
        result: [],
        remove: false
      }).result.join('');

      if (changed) {
        var pathToUpdate = updateInline ? pathToCheckAndUpdate : path.join(destination, fileName); // if (fs.existsSync(pathToUpdate)) {

        fs.writeFileSync(pathToUpdate, source, {
          flag: 'w+'
        }); // }

        if (debug) {
          console.log('wrote to file', pathToUpdate);
        }
      }
    }
  });
  console.log('Successfully env to all the files ');
}

function T(source) {
  var options = loaderUtils.getOptions(this);
  var exclude = options.exclude,
      plugins = options.plugins,
      enable = options.enable,
      _options$debug = options.debug,
      debug = _options$debug === void 0 ? false : _options$debug,
      sourceType = options.sourceType;

  if (enable) {
    var ast = parser.parse(source, {
      sourceType: sourceType || 'module',
      plugins: plugins || ['jsx']
    });
    var codalo = source;
    traverse(ast, {
      enter: function enter(path) {
        // Why doing the ast traverse ?
        // I want to be sure , that the modification is happening
        // only for process.env member Identifiers. There is no
        // way to guarantee that with string parsing.
        if (BabelTypeModule.isMemberExpression(path.node)) {
          var generatedCode = generator(path.node);
          var pathArr = toPath(generatedCode.code);

          if (pathArr[0] === 'process' && pathArr[1] === 'env' && pathArr.length === 3) {
            var paramName = '';

            if (pathArr[2]) {
              paramName = pathArr[2].trim();
            }

            if (debug) {
              console.log("Path\xA0found\xA0in\xA0the\xA0code\xA0is\xA0:\xA0" + pathArr);
              console.log("Path\xA0name\xA0is\xA0:\xA0" + paramName);
            }

            var isBlackList = exclude ? exclude.findIndex(function (blackListedItem) {
              return blackListedItem === paramName;
            }) !== -1 : false;

            if (!isBlackList) {
              try {
                var MM = BabelTypeModule.templateLiteral([BabelTypeModule.templateElement({
                  raw: "$$_INTERNAL__process.env." + paramName + "MATH_RANDOM_START",
                  cooked: "$$_INTERNAL__process.env." + paramName + "MATH_RANDOM_START"
                }), BabelTypeModule.templateElement({
                  raw: 'MATH_RANDOM_END$$_INTERNAL__',
                  cooked: 'MATH_RANDOM_END$$_INTERNAL__'
                })], [BabelTypeModule.callExpression(BabelTypeModule.memberExpression(BabelTypeModule.identifier('Math'), // most important part, for confusing //babel not to replace the values in the code
                BabelTypeModule.identifier('random'), false), [])]);
                path.replaceWith(MM);
              } catch (e) {
                console.log('error is ===>', e);
              }

              codalo = generator(ast).code;
            }
          }
        }
      }
    });
    return codalo;
  }

  return source;
}

exports.default = T;
exports.injectEnv = injectEnv;
//# sourceMappingURL=envvarprep-loader.cjs.development.js.map
